<?php
/**
 * @category      module
 *
 * @author        eKomi
 *
 * @see           https://www.ekomi.de
 *
 * @copyright (C) eKomi 2001-2017
 */

use OxidEsales\Eshop\Core\Registry;

/**
 * Class ekomiIntegrationModule.
 *
 * Handles module setup, provides additional tools and module related helpers.
 *
 * @codeCoverageIgnore
 */
class ekomiIntegrationModule extends oxModule
{
    /**
     * Module id.
     */
    const MODULE_ID = 'ekomiintegration';

    /**
     * Order Statuses.
     */
    const ORDER_STATUS_CREATED = 'ORDERFOLDER_NEW';
    const ORDER_STATUS_SHIPPED = 'Shipped';

    /**
     * Product Identifier.
     */
    const ARTICLE_NUMBER = 'ARTNUM';
    const ARTICLE_EAN = 'EAN';
    const ARTICLE_DIST_EAN = 'DISTEAN';
    const ARTICLE_MPN = 'MPN';

    /**
     * Exports methods.
     */
    const EXPORT_METHOD_HOOK = 'hook';
    const EXPORT_METHOD_CRON = 'cron';

    /**
     * Configuration to save last run cron timestamp.
     */
    const LAST_EXPORTED_ORDER_TIMESTAMP = 'ekomiConfigLastExportedOrderTimestamp';

    /**
     * Orders Limit.
     */
    const MAX_ORDERS_LIMIT = 50;

    /**
     * Plugin name in Plugins-dashboard.
     */
    const PD_PLUGIN_NAME = 'oxid';

    /**
     * Response message types.
     */
    const MESSAGE_TYPE_SUCCESS = 'success';
    /**
     * The Url to validate shop.
     */
    const URL_GET_SETTINGS = 'https://api.ekomi.de/v3/getSettings';

    /**
     * The URL where the order data is sent.
     */
    const URL_TO_SEND_DATA = 'https://plugins-dashboard.ekomiapps.de/api/v1/order';

    /**
     * ekomiIntegrationModule constructor.
     * Sets main module data and load additional data.
     *
     * @param string $sModuleTitle
     * @param string $sModuleDescription
     */
    public function __construct($sModuleTitle = 'eKomi Integration', $sModuleDescription = 'eKomi Integration Module')
    {

        $this->setModuleData(
            array(
                'id' => self::MODULE_ID,
                'title' => $sModuleTitle,
                'description' => $sModuleDescription,
            )
        );

        $this->load(self::MODULE_ID);

        oxRegistry::set('ekomiIntegrationModule', $this);
    }

    /**
     * Returns the module instance.
     *
     * @return EkomiIntegrationEvents
     */
    public static function getInstance()
    {
        return oxRegistry::get('ekomiIntegrationModule');
    }

    /**
     * Module activation script.
     */
    public static function onActivate()
    {
        oxRegistry::get('oxUtilsView')->addErrorToDisplay(new oxException('Please save the module settings.'));

        self::clearTmp();

        return true;
    }

    /**
     * Fetches plugin configuration data.
     *
     * @return array
     */
    public static function getConfigurationData()
    {
        $configurations = array(
            'shopId' => ekomiIntegrationModule::getSetting('ShopId'),
            'shopSecret' => ekomiIntegrationModule::getSetting('ShopPw'),
            'mode' => ekomiIntegrationModule::getSetting('Mode'),
            'reviews' => ekomiIntegrationModule::getSetting('ProductReviews'),
            'status' => ekomiIntegrationModule::getSetting('Status'),
            'exportMethod' => ekomiIntegrationModule::getSetting('ExportMethod'),
            'turnaroundTime' => ekomiIntegrationModule::getSetting('TurnaroundTime'),
            'productIdentifier' => ekomiIntegrationModule::getSetting('ProductIdentifier'),
            'excludeProducts' => ekomiIntegrationModule::getSetting('ExcludeProducts'),
            'showPrcWidget' => ekomiIntegrationModule::getSetting('ShowPrcWidget'),
            'prcWidgetToken' => ekomiIntegrationModule::getSetting('PrcWidgetToken'),
            'lastExportedOrderTimestamp' => oxRegistry::getConfig()->getSystemConfigParameter(
                self::LAST_EXPORTED_ORDER_TIMESTAMP
            ),
            'termsAndConditions' => ekomiIntegrationModule::getSetting('TermsAndConditions'),
        );

        return $configurations;
    }

    /**
     * Updates last order updated date time.
     *
     * @param string $timestamp
     */
    public static function updateLastExportedOrderTimestamp($timestamp)
    {
        oxRegistry::getConfig()->saveSystemConfigParameter('str', self::LAST_EXPORTED_ORDER_TIMESTAMP, $timestamp);
    }

    /**
     * Verifies eKomi shop id and password.
     *
     * @param int    $shopId       The shop interface id
     * @param string $shopPassword The Shop interface password
     *
     * @return array
     */
    public static function verifyAccount($shopId, $shopPassword)
    {
        if (!empty($shopId) && !empty($shopPassword)) {
            $apiUrl = self::URL_GET_SETTINGS;
            $apiUrl .= "?auth={$shopId}|{$shopPassword}";
            $apiUrl .= '&version=cust-1.0.0&type=request&charset=iso&app=PD-Oxid';
            $helper = new ekomiIntegrationHelper();
            $serverOutput = $helper->doCurl($apiUrl, 'GET');
            $response = array('success' => true, 'message' => 'Plug-in activated');
            if ('Access denied' == $serverOutput) {
                $response = array('success' => false, 'message' => 'Invalid Shop ID or Secret');
            }

            return $response;
        } else {
            return array('success' => false, 'message' => 'Shop ID and Password required');
        }
    }

    /**
     * Module deactivation script.
     */
    public static function onDeactivate()
    {
        self::clearTmp();
    }

    /**
     * Deactivates the Plugin manually if credentials ae not correct.
     *
     * @return bool
     */
    protected static function _deactivatePlugin()
    {
        $sModule = self::MODULE_ID;
        $oModule = oxNew('oxModule');
        if (!$oModule->load($sModule)) {
            oxRegistry::get('oxUtilsView')->addErrorToDisplay(new oxException('EXCEPTION_MODULE_NOT_LOADED'));

            return;
        }
        /** @var oxModuleCache $oModuleCache */
        $oModuleCache = oxNew('oxModuleCache', $oModule);
        /** @var oxModuleInstaller $oModuleInstaller */
        $oModuleInstaller = oxNew('oxModuleInstaller', $oModuleCache);

        if ($oModuleInstaller->deactivate($oModule)) {
            return true;
        }

        return false;
    }

    /**
     * Cleans temp folder content.
     *
     * @param string $sClearFolderPath Sub-folder path to delete from. Should be a full, valid path inside temp folder.
     *
     * @return bool
     */
    public static function clearTmp($sClearFolderPath = '')
    {
        $sFolderPath = self::_getFolderToClear($sClearFolderPath);
        $hDirHandler = opendir($sFolderPath);

        if (!empty($hDirHandler)) {
            while (false !== ($sFileName = readdir($hDirHandler))) {
                $sFilePath = $sFolderPath . DIRECTORY_SEPARATOR . $sFileName;
                self::_clear($sFileName, $sFilePath);
            }

            closedir($hDirHandler);
        }

        return true;
    }

    /**
     * Gets CMS snippet content by identified id.
     *
     * @param string $sIdentifier
     * @param bool   $blNoHtml
     *
     * @return string
     */
    public function getCmsContent($sIdentifier, $blNoHtml = true)
    {
        $sValue = '';

        /** @var oxContent|oxI18n $oContent */
        $oContent = oxNew('oxContent');
        $oContent->loadByIdent(trim((string)$sIdentifier));

        if ($oContent->oxcontents__oxcontent instanceof oxField) {
            $sValue = (string)$oContent->oxcontents__oxcontent->getRawValue();
            $sValue = (empty($blNoHtml) ? $sValue : nl2br(strip_tags($sValue)));
        }

        return $sValue;
    }

    /**
     * Gets module setting value.
     *
     * @param string $sModuleSettingName module setting parameter name (key)
     * @param bool   $blUseModulePrefix  if True - adds the module settings prefix, if False - not
     *
     * @return string
     */
    public static function getSetting($sModuleSettingName, $blUseModulePrefix = true)
    {
        if ($blUseModulePrefix) {
            $sModuleSettingName = 'ekomiConfig' . (string)$sModuleSettingName;
        }

        return oxRegistry::getConfig()->getConfigParam((string)$sModuleSettingName);
    }


    /**
     * Gets module path.
     *
     * @return string full path to the module directory
     */
    public function getPath()
    {
        return oxRegistry::getConfig()->getModulesDir() . 'ekomi/ekomiintegration/';
    }

    /**
     * Checks if provided path is inside eShop `tpm/` folder or use the `tmp/` folder path.
     *
     * @param string $sClearFolderPath
     *
     * @return string
     */
    protected static function _getFolderToClear($sClearFolderPath = '')
    {
        $sTempFolderPath = (string)oxRegistry::getConfig()->getConfigParam('sCompileDir');

        if (!empty($sClearFolderPath) and (false !== strpos($sClearFolderPath, $sTempFolderPath))) {
            $sFolderPath = $sClearFolderPath;
        } else {
            $sFolderPath = $sTempFolderPath;
        }

        return $sFolderPath;
    }

    /**
     * Checks if resource could be deleted, then delete it's a file or call recursive folder deletion if it's a directory.
     *
     * @param string $sFileName
     * @param string $sFilePath
     */
    protected static function _clear($sFileName, $sFilePath)
    {
        if (!in_array($sFileName, array('.', '..', '.gitkeep', '.htaccess'))) {
            if (is_file($sFilePath)) {
                @unlink($sFilePath);
            } else {
                self::clearTmp($sFilePath);
            }
        }
    }
}
