<?php

/**
 * PAYONE OXID Connector is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * PAYONE OXID Connector is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with PAYONE OXID Connector.  If not, see <http://www.gnu.org/licenses/>.
 *
 * @see           http://www.payone.de
 *
 * @copyright (C) Payone GmbH
 *
 * @version       OXID eShop CE
 */
class ekomiExportOrdersHelper extends oxBase
{
    /**
     * Flags if shop uses registry.
     *
     * @var static boolean
     */
    protected static $_blUseRegistry = null;

    /**
     * @var ekomiIntegrationHelper|null
     */
    protected $_ekomiHelper = null;

    /**
     * Building essential stuff.
     */
    public function __construct()
    {
        parent::__construct();

        $this->_ekomiHelper = new ekomiIntegrationHelper();
    }

    /**
     * Exports orders.
     *
     * @param bool   $orderId
     * @param string $orderStatus
     *
     * @throws \OxidEsales\Eshop\Core\Exception\DatabaseConnectionException
     * @throws \OxidEsales\Eshop\Core\Exception\DatabaseErrorException
     */
    public function exportOrder($orderId = false, $orderStatus)
    {
        $configurations = ekomiIntegrationModule::getConfigurationData();
        if ($orderId && ekomiIntegrationModule::EXPORT_METHOD_HOOK === $configurations['exportMethod']
            && $orderStatus === $configurations['status']
        ) {
            $this->exportOrderHook($orderId, $configurations);
        }
    }

    /**
     * Sends order data to eKomi plugins-dashboard.
     *
     * @param string $orderId
     * @param array  $configurations
     *
     * @return string
     *
     * @throws \OxidEsales\Eshop\Core\Exception\DatabaseConnectionException
     * @throws \OxidEsales\Eshop\Core\Exception\DatabaseErrorException
     */
    public function exportOrderHook($orderId, $configurations)
    {
        $activeShop = $this->getConfig()->getActiveShop();
        $oxShopId = $activeShop->oxshops__oxid->value;
        $oOrder = $this->_fetchOrder($orderId, $oxShopId);

        return $this->sendOrderDataToEkomi($oOrder, $configurations, $activeShop);
    }

    /**
     * Runs after every 24 hour.
     *
     * @param $configurations
     *
     * @return string
     *
     * @throws \OxidEsales\Eshop\Core\Exception\DatabaseConnectionException
     * @throws \OxidEsales\Eshop\Core\Exception\DatabaseErrorException
     */
    public function exportOrderCron($configurations)
    {
        $activeShop = $this->getConfig()->getActiveShop();
        $oxShopId = $activeShop->oxshops__oxid->value;
        $oOrdersData = $this->_fetchAllOrders($configurations, $oxShopId);
        $oxTimestamp = '';
        $successCount = 0;
        foreach ($oOrdersData as $key => $order) {
            $response = $this->sendOrderDataToEkomi($order, $configurations, $activeShop);
            $responseObject = json_decode($response);
            if (ekomiIntegrationModule::MESSAGE_TYPE_SUCCESS === $responseObject->status->type) {
                ++$successCount;
            }

            $oxTimestamp = $order['OXTIMESTAMP'];
        }

        if (!empty($oxTimestamp)) {
            ekomiIntegrationModule::updateLastExportedOrderTimestamp($oxTimestamp);
        }

        $totalOrders = count($oOrdersData);

        return "{$successCount} out of {$totalOrders} Orders sent successfully.";
    }

    /**
     * Sends order data to eKomi.
     *
     * @param array $order
     *
     * @return string
     *
     * @throws \OxidEsales\Eshop\Core\Exception\DatabaseConnectionException
     * @throws \OxidEsales\Eshop\Core\Exception\DatabaseErrorException
     */
    public function sendOrderDataToEkomi($order, $configurations, $activeShop)
    {
        if (!empty($order)) {
            $boundary = md5(time());
            $httpHeader = array('ContentType:multipart/form-data;boundary=' . $boundary);

            $order['OXCUSTNR'] = '';
            $order['OXBILLCOUNTRYISO2'] = '';
            $order['OXBILLCOUNTRYISO3'] = '';
            $order['OXDELCOUNTRYIDISO2'] = '';
            $order['OXDELCOUNTRYIDISO3'] = '';

            $oUser = oxNew('oxuser');
            $oUser->load($order['OXUSERID']);
            if ($oUser) {
                $order['OXCUSTNR'] = $oUser->oxuser__oxcustnr->value;
            }

            $oCountry = oxNew('oxcountry');
            $oCountry->load($order['OXBILLCOUNTRYID']);
            if ($oCountry) {
                $order['OXBILLCOUNTRYISO2'] = $oCountry->oxcountry__oxisoalpha2->value;
                $order['OXBILLCOUNTRYISO3'] = $oCountry->oxcountry__oxisoalpha3->value;
            }

            $oCountry = oxNew('oxcountry');
            $oCountry->load($order['OXDELCOUNTRYID']);
            if ($oCountry) {
                $order['OXDELCOUNTRYIDISO2'] = $oCountry->oxcountry__oxisoalpha2->value;
                $order['OXDELCOUNTRYIDISO3'] = $oCountry->oxcountry__oxisoalpha3->value;
            }

            $postFields = $this->_prepareRequiredFields($order, $configurations, $activeShop);
            if ((bool)$configurations['termsAndConditions']) {
                return $this->_ekomiHelper->doCurl(
                    ekomiIntegrationModule::URL_TO_SEND_DATA,
                    'PUT',
                    $httpHeader,
                    json_encode($postFields)
                );
            }
        }
    }

    /**
     * Fetches Required fields for API.
     *
     * @param array  $oOrder
     * @param array  $configurations
     * @param object $activeShop
     *
     * @return array
     *
     * @throws \OxidEsales\Eshop\Core\Exception\DatabaseConnectionException
     * @throws \OxidEsales\Eshop\Core\Exception\DatabaseErrorException
     */
    protected function _prepareRequiredFields($oOrder, $configurations, $activeShop)
    {
        $oOrder['senderName'] = $activeShop->oxshops__oxname->value;
        $oOrder['senderEmail'] = $activeShop->oxshops__oxinfoemail->value;
        $oOrder['products'] = $this->_getProductsData($oOrder['OXID'], $oOrder['OXLANG']);
        $postFields = array(
            'shop_id' => $configurations['shopId'],
            'interface_password' => $configurations['shopSecret'],
            'order_data' => $oOrder,
            'mode' => $configurations['mode'],
            'product_reviews' => $configurations['reviews'],
            'product_identifier' => $configurations['productIdentifier'],
            'exclude_products' => $configurations['excludeProducts'],
            'plugin_name' => ekomiIntegrationModule::PD_PLUGIN_NAME,
        );

        return $postFields;
    }

    /**
     * Gets products data.
     *
     * @param string $oxOid
     * @param int    $languageId
     *
     * @return array
     *
     * @throws \OxidEsales\Eshop\Core\Exception\DatabaseConnectionException
     * @throws \OxidEsales\Eshop\Core\Exception\DatabaseErrorException
     */
    protected function _getProductsData($oxOid, $languageId)
    {
        $orderProducts = $this->_getArticles($oxOid, $languageId);
        foreach ($orderProducts as $key => $product) {
            $shopUrl = $this->getConfig()->getShopUrl();
            $product['imageUrl'] = utf8_decode($shopUrl . 'out/pictures/master/product/1/' . $product['OXPIC1']);
            $product['canonicalUrl'] = utf8_decode($shopUrl . $product['OXSEOURL']);
            $orderProducts[$key] = $product;
        }

        return $orderProducts;
    }

    /**
     * Fetches all orders from database.
     *
     * @param array $configurations
     * @param int   $oxShopId
     *
     * @return mixed
     *
     * @throws Exception
     */
    protected function _fetchAllOrders($configurations, $oxShopId)
    {
        $dateLimit = $this->_ekomiHelper->getDateTimeLimit($configurations);
        $oDb = $this->_getDB();
        /**
         * Prepare query to fetch orders.
         */
        $sSelect = 'SELECT * FROM `oxorder`';
        $sSelect .= "WHERE `OXSHOPID` = {$oDb->quote($oxShopId)} AND `OXTIMESTAMP` > {$oDb->quote($dateLimit)}";
        if (ekomiIntegrationModule::ORDER_STATUS_SHIPPED == $configurations['status']) {
            $sSelect .= ' AND `OXSENDDATE` != ' . $oDb->quote('0000-00-00 00:00:00');
        } else {
            $sSelect .= ' AND `OXFOLDER` = ' . $oDb->quote($configurations['status']);
        }

        $sSelect .= ' ORDER BY `OXTIMESTAMP` ASC LIMIT ' . ekomiIntegrationModule::MAX_ORDERS_LIMIT;

        return $oDb->getAll($sSelect);
    }

    /**
     * Fetches orders.
     *
     * @param int $orderId
     * @param int $oxShopId
     *
     * @return array
     *
     * @throws \OxidEsales\Eshop\Core\Exception\DatabaseConnectionException
     * @throws \OxidEsales\Eshop\Core\Exception\DatabaseErrorException
     */
    protected function _fetchOrder($orderId, $oxShopId)
    {
        $oDb = $this->_getDB();

        /**
         * Prepare query to fetch orders.
         */
        $sSelect = 'SELECT * FROM `oxorder`';
        $sSelect .= " WHERE `OXSHOPID` = {$oDb->quote($oxShopId)} AND `OXID` = {$oDb->quote($orderId)}";
        $oOrders = $oDb->getAll($sSelect);

        if ($oOrders && !empty($oOrders)) {
            return $oOrders[0];
        }

        return array();
    }

    /**
     * Returns assigned orderarticles for order.
     * .
     *
     * @param string $oxOid
     * @param int    $languageId
     *
     * @return array
     *
     * @throws \OxidEsales\Eshop\Core\Exception\DatabaseConnectionException
     * @throws \OxidEsales\Eshop\Core\Exception\DatabaseErrorException
     */
    protected function _getArticles($oxOid, $languageId)
    {
        $oDb = $this->_getDB();
        $sSelect = 'SELECT oxoa.*, `oxseo`.`OXSEOURL`  FROM `oxorderarticles` oxoa'
            . ' INNER JOIN `oxseo`'
            . " ON oxoa.`OXARTID` = `oxseo`.`OXOBJECTID` AND `oxseo`.`OXLANG` = {$languageId}"
            . " WHERE oxoa.`oxorderid` = '{$oxOid}'";
        $oArticles = $oDb->getAll($sSelect);

        return $oArticles;
    }

    /**
     * Gets DB instance.
     *
     * @return \OxidEsales\Eshop\Core\Database\Adapter\DatabaseInterface
     *
     * @throws \OxidEsales\Eshop\Core\Exception\DatabaseConnectionException
     */
    protected function _getDB()
    {
        return oxDb::getDb(oxDb::FETCH_MODE_ASSOC);
    }

    /**
     * Returns a factory instance of given object.
     *
     * @param string $sName
     *
     * @return object
     */
    public function getFactoryObject($sName)
    {
        return oxNew($sName);
    }
}
